/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

/** @file pam_modutil.h
 * @brief Utility functions for PAM module
 *
 * The file implements some auxiliary functions to PAM module. The implementation
 * bases on the code of PAM utility, which observes the GPL. The auxiliary
 * functions here is mainly to support the lower version. If the version of Linux
 * PAM is higher than 0.99, the file and functions are unnecessary. 
 */

#ifndef _PAM_MODUTIL_H_
#define _PAM_MODUTIL_H_

#include <sys/types.h>
#include <security/pam_modules.h>



#define PWD_INITIAL_LENGTH     0x100
#define PWD_ABSURD_PWD_LENGTH  0x8000

/** PAM module cleanup function
 * @param pamh the handler of PAM module
 * @param data The data to clean
 * @param error_status The error code to return
 */
extern void
pam_modutil_cleanup(pam_handle_t * pamh, void *data, int error_status);

/** PAM security fuction - getpwnam
 * @param pamh The handler of PAM module
 * @param user The account name 
 * @return a pointer of struct passwd if success, or NULL
 */
extern struct passwd *pam_modutil_getpwnam(pam_handle_t * pamh,
					   const char *user);
/** PAM security function -  getpwuid
 * @param pamh The handler of PAM module
 * @param uid the uid of account
 * @return a pointer of struct passwd if success, or NULL
 */
extern struct passwd *pam_modutil_getpwuid(pam_handle_t * pamh, uid_t uid);

/** PAM security function -  getgrnam
 * @param pamh The handler of PAM module
 * @param group the group name
 * @return a pointer of struct group if success, or NULL
 */
extern struct group *pam_modutil_getgrnam(pam_handle_t * pamh,
					  const char *group);
/** PAM security function - getgrgid
 * @param pamh The handler of PAM moudule
 * @param gid The gid of group needed to be fetched information
 * @return a pointer of struct group if success, or NULL
 */
extern struct group *pam_modutil_getgrgid(pam_handle_t * pamh, gid_t gid);


/** Judge whether user is in group
 * @param pamh handler of PAM 
 * @param user string pointer of user name
 * @param group string pointer of group name
 *
 * @return 1 if user is in the group, or 0.
 */
extern int
pam_modutil_user_in_group_nam_nam(pam_handle_t * pamh,
				  const char *user, const char *group);


#endif				/* _PAM_MODUTIL_H_ */
