/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */


/** @file xos_cgrp.h
 * @brief client interfaces for cgroups service
 *
 * The file aims at implementing the AMS client codes, which provide connecting and
 * transmitting interfaces with AMS server.
 */

#ifndef _XOS_CGRP_H_
#define _XOS_CGRP_H_

#include <stdio.h>
#include <stdlib.h>

/** call the server to add a new pid in container
 * @param subsys the subsystem name 
 * @param cnt the container name to control given pid
 * @param pid the will-be inserted pid
 * @return 0 if successfully, or !0
 * */
extern int hd_cgroup_insertpid(char *subsys, char *cnt, int pid);
extern int hd_cgroup_insertpid_internal(int sockfd, char *subsys, char *cnt, int pid);

/** call the server to delete given pid from given container
 * @param cnt the container holding the given deleted pid
 * @param pid the will-be deleted pid
 * @return 0 if successfully, or !0
 * */
extern int hd_cgroup_delpid(char *cnt, int pid);
extern int hd_cgroup_delpid_internal(int sockfd, char *cnt, int pid);

/** call the server to count the amount of pids for given container
 * @param cnt the container name
 * @param pid (unused)
 * @return >=0 of total amount of processes in container, or <0
 * */
extern int hd_cgroup_countpid(char *cnt, int pid);
extern int hd_cgroup_countpid_internal(int sockfd, char *cnt, int pid);

/** call the server to mount the cgroupfs for given subsystem
 * @param subsys the subsystem needs to be mounted
 * @return 0 if successfully, or !0
 * */
extern int hd_cgroup_mountfs(char *subsys);
extern int hd_cgroup_mountfs_internal(int sockfd, char *subsys);

/** call the server to umount cgroupfs for given subsystem
 * @param subsys the subsystem name
 * @return 0 if successfully, or !0
 * */
extern int hd_cgroup_umountfs(char *subsys);
extern int hd_cgroup_umountfs_internal(int sockfd, char *subsys);

/** call the server to create a new container
 * @param name the container name would be created
 * @param subsys the container will be controlled in given subsystem
 * @return 0 if successfully, or !0
 * */
extern int hd_cgroup_create(char *name, char *subsys);
extern int hd_cgroup_create_internal(int sockfd, char *name, char *subsys);

/** call the server to destroy an existing container
 * @param name the will-be destroyed container name
 * @param subsys the subsystem controlling the container
 * @return 0 if successfully, or !0
 * */
extern int hd_cgroup_destroy(char *name, char *subsys);
extern int hd_cgroup_destroy_internal(int sockfd, char *name, char *subsys);

#endif				/* _XOS_CGRP_H_ */
