/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

/** @file xos_protocol.h
 * @brief the protocol used in communication between AMS client and server.
 *
 * Notice: the protocol is not secure one.
 * 
 * */
#ifndef _XOS_PROTOCOL_H_
#define _XOS_PROTOCOL_H_

#include "xos.h"
#include "xos_policy_rule.h"
#include "xos_dbaux.h"

#define	PACKAGE_MAXLEN	AMS_PACKAGE_MAXLEN

#define MSG_END         "$quit$"

typedef enum _opttype {
	PTL_OT_NULL,		/* 0 */
	PTL_OT_USERMAPPING,	/* 1 */
	PTL_OT_CLEARACCT,	/* 2 */
	PTL_OT_CLEARGRP,	/* 3 */
	PTL_OT_CLEARACTGRP,	/* 4 */
	PTL_OT_GETBYACTGRP,	/* 5 */
	PTL_OT_GETGRP,		/* 6 */
	PTL_OT_CLEARUIDGID,	/* 7 */
	PTL_OT_GETBYUIDGID,	/* 8 */
	PTL_OT_SETUPQUOTA,	/* 9 */
	PTL_OT_CLEARQUOTA,	/* 10 */
	PTL_OT_SAVETMPINFO,	/* 11 */
	PTL_OT_GETTMPINFO,	/* 12 */
	PTL_OT_RULEPUT,		/* 13 */
	PTL_OT_GETRULEBYKEY,	/* 14 */
	PTL_OT_GETBYHANDLER,	/* 15 */
	PTL_OT_DELETERULE,	/* 16 */
	PTL_OT_CLEARRULES,	/* 17 */
	PTL_OT_SEARCHRULE,	/* 18 */
	PTL_OT_ACREJECT,	/* 19 */
	PTL_OT_ENFORCERULE,	/* 20 */
        PTL_OT_CHUSERINFO,     	/* 21 */
	PTL_OT_OPTMOUNTCNT,     /* 22 */
        PTL_OT_CGRPADDPID,      /* 23 */
        PTL_OT_CGRPDELPID,      /* 24 */
        PTL_OT_CGRPCOUNTPID,    /* 25 */
        PTL_OT_MOUNTFS,         /* 26 */
        PTL_OT_UMOUNTFS,        /* 27 */
        PTL_OT_CNTCRT,          /* 28 */
        PTL_OT_CNTDESTY,         /* 29 */
	PTL_OT_MOUNTHOME,        /* 30 */
	PTL_OT_UMOUNTHOME,       /* 31 */
} OPTTYPE;


/** AMS client interfaces */


/** Packing function according protocol in AMS client
 * @param message the buffer to be stored packed data.
 * @param optcode the operation code
 * @param mapped_dn the mapped DN information
 * @param mapped_vo the mapped VO information
 * @param mapped_role the mapped ROLE information
 * @param mapped subgroups the mapped SUBGROUPS information
 * @return zero if success, or -1
 * */

int amsclient_packetmsg(char *message, OPTTYPE optcode, char *mapped_dn,
			char *mapped_vo, char *mapped_role,
			char *mapped_subgroups);

/** unpacking function for AMS client
 * @param message the buffer to be unpacked
 * @param gpwd the pointer of struct GPASSWD should be preallocated
 * @param ggrp the pointer of struct GGROUPS should be preallocated
 * @param ret the return value of AMS server finishing task
 * @return zero if success, or -1
 * */

int amsclient_unpacketmsg(char *message, GPASSWD * gpwd,
			  GGROUPS * ggrp, int *ret);

/** unpacking function for temporary information
 * @param message the buffer to be unpacked
 * @param tmpinfo the buffer of temporary information
 * @param ret the returned value of AMS server
 * @return zero if the operation is successful, or non-zero.
 * */
int amsclient_unpacketmsg_tmpinfo(char *message, char *tmpinfo, int *ret);

/** open the connection with AMS server.
 * @return file discription of socket if successful, or non-zero.
 * */
int amsclient_connect_open();

/** Send query message to AMS server
 * The function majors in transmition with AMS server
 * @param sockfd the file discription of socket opened by amsclient_connect_open().
 * @param message the message buffer to exchange
 * @param message_len the length of exchanged message
 * @return zero if success, or -1
 * */

int amsclient_connect_query(int sockfd, char *message, int *message_len);

/** close the connection with AMS server.
 * @param sockfd the file discription of socket opened by amsclient_connect_open().
 * @return zero if successful, or non-zero.
 * */
int amsclient_connect_close(int sockfd);


/** unpacking function for rule in AMS cleint
 * @param message the buffer to be unpacked
 * @param xrule the rule pointer will be holded the unpacked data.
 * @param handler the handler of unpacked rule.
 * @param ret the returned value of AMS internal operation.
 * @return zero if the operation is successful, or non-zero.
 * */
int amscient_unpacketmsg_rule(char *message, struct rule_t *xrule,
		             char *handler, int *ret);


/* AMS server interfaces */


/** packing function for AMS server.
 * @param message the buffer to be stored packed data.
 * @param gpwd the returned GPASSWD to be packed
 * @param ggrp the returned GGROUPS to be packet
 * @param ret the returened value of AMS internal operation.
 * @returen zero if the operation is successful, or non-zoro.
 * */
int amsserver_packetmsg(char *message, GPASSWD * gpwd, GGROUPS * ggrp,
			int ret);

/** packing function for  temporary data in AMS server
 * @param message the buffer to be packed
 * @param tmpinfo the temp information to be packed.
 * @param ret the returened value of AMS internal operation.
 * @return zero if the operation is successful, or non-zoro.
 *
 * */
int amsserver_packetmsg_tmpinfo(char *message, char *tmpinfo, int ret);

/** packing function for rule in AMS server
 * @param message the buffer to be stored packed data.
 * @param xrule the rule data will be packed
 * @pram handler the rule handler to be packed
 * @param ret the operation returned value.
 * @return zero if the operation is successful, or non-zoro.
 * */

int amsserver_packetmsg_rule(char *message, struct rule_t *xrule,
		             char *handler, int ret);


/*  packet function */


/** packing fucntion for rule 
 * @param message the buffer to be stored packed data.
 * @param xrule the rule data to be packed.
 * @return zero if the operation is successful, or non-zoro.
 * */
int protocol_packetrule(char *message, struct rule_t *xrule);

/** unpacking the rule message
 * @param message the buffer holding the data to be unpacked
 * @param xrule the rule buffer to be returned.
 * @return zero if the operation is successful, or non-zoro.
 * */
int protocol_unpacketrule(char *message, struct rule_t *xrule);

/** packing function for subject
 * @param the message to be stored packed data
 * @pram sub the pointer of subject to be packed
 * @return zero if the operation is successful, or non-zoro. 
 * */
int protocol_packetsubject(char *message, struct subject_t *sub);

/** unpacking function for subject
 * @param message the message to be stored packed data
 * @param sub the pointer to returned subject data.
 * @return zero if the operation is successful, or non-zoro. 
 * */
int protocol_unpacketsubject(char *message, struct subject_t *sub);



#endif				/* _PROTOCOL_H_ */
